function WindowMessage(windowId, contentId, elementType, elementId, controlType, arg) {
	this.messagetype = "window";

	this.windowId = windowId;
	this.contentId = contentId;
	this.elementType = elementType;
	this.elementId = elementId;
	this.controlType = controlType;
	this.arg = arg;
};

function sendMessage(element, controlType, arg) {
	var message = null;
	if (element.hasClass("window")) {
		message = new WindowMessage(element.attr("data-id"), "", "", "", controlType, arg);
	} else if (element.hasClass("window-content")) {
		var window = element.closest(".window");
		message = new WindowMessage(window.attr("data-id"), element.attr("data-id"), "", "", controlType, arg);
	} else if (element.hasClass("window-item")) {
		var content = element.closest(".window-content");
		var window = content.closest(".window");
		message = new WindowMessage(window.attr("data-id"), content.attr("data-id"), element.attr("data-element"), element.attr("data-id"), controlType, arg);
	}
	if (message) windowConnection._send(message);
}

var windowConnection;

function onWindowElementReady(element) {
	$('.window-text', element).each(function () {
		if ($(this).attr("data-type") === "error") $(this).addClass("ui-state-error-text");
		else if ($(this).attr("data-type") === "plain") $(this).html("<pre>" + $(this).html() + "</pre>");
		
		if ($(this).attr("data-state") === "error") $(this).addClass("ui-state-error-text");
		else if ($(this).attr("data-state") === "success") $(this).addClass("ui-state-success-text");
		
		// Kaamea viritys. Tehdään |-|www.com|-| -muotoisista tekstin pätkistä linkkejä.
		$(this).html($(this).html().replace(/\|-\|(.{0,100})\|-\|/g, "<a href='$1' target='_blank'>$1</a>"));
	});
	
	$('.window-progressbar', element).each(function () {
		$(this).progressbar({ value: $(this).attr("data-indeterminate") === "true" ? false : 0 });
	});

	$('.window-item[data-busy="true"]', element).each(function () {
		var busy = $(".window-busy", element);
		busy.show();
		rotate(busy);
	});

	$('.window-item[data-disabled="true"]', element).each(function () {
		$(this).addClass("ui-state-disabled").prop("disabled", true);
	});
	
	$('.window-input:not(.window-grid)', element).each(function () {
		if ($(this).attr("data-state") === "error") $(this).addClass("ui-state-error");
		else if ($(this).attr("data-state") === "highlight") $(this).addClass("ui-state-highlight");
		else if ($(this).attr("data-state") === "success") $(this).addClass("ui-state-success");
	});

	$('.window-scheduleeditor', element).each(function () {
		var table = $("<table></table>");
		
		function pad(num, size) {
			var s = "000000000" + num;
			return s.substr(s.length-size);
		}
		
		var timedayrow = $("<tr></tr>").append("<td>" + $(this).attr("data-timeday") + "</td>");
		for (var hour = 0; hour < 24; ++hour) timedayrow.append($("<td><input class='ui-state-default window-schedule-editor-hour' type='button' data-hour='" + hour + "' value='" + pad(hour, 2) + "-' /></td>"));
		table.append(timedayrow);
		
		for (var day = 0; day < 7; ++day) {
			var row = $("<tr></tr>");
			var daybutton = $("<button>" + $(this).attr("data-day" + day)  + "</button>").addClass("ui-state-default").addClass("window-schedule-editor-day").attr("data-day", day);
			row.append($("<td></td>").append(daybutton));
			for (var hour = 0; hour < 24; ++hour) {
				var selected = $(this).attr("data-value")[hour * 7 + day] === 'A';
				var button = $("<div/>").addClass('window-schedule-editor-item').attr("data-selected", selected ? "true" : "false").attr("data-day", day).attr("data-hour", hour);
				row.append($("<td></td>").append(button));
			}
			table.append(row);
		}

		$(this).append(table);
	});
	
	$('.window-multichoice').each(function () {
		$(this).css("height", parseInt($(this).attr("data-height")));
		$(this).css("overflow-y", "scroll");
		var w = parseInt($(this).attr("data-width"));
		$(this).css("width", w === -1 ? $(this).width() : w);
	});
	
	$('.window-slider', element).slider({
		change: function (event, ui) {
			if (!event.originalEvent) return; // Testataan oliko käyttäjän aiheuttama eventti
			sendMessage($(this), "change", ui.value);
		},
		create: function (event, ui) {
			$(this).slider("option", "min", parseInt($(this).attr("data-min")));
			$(this).slider("option", "max", parseInt($(this).attr("data-max")));
			$(this).slider("option", "value", parseInt($(this).attr("data-value")));
			$(this).slider("option", "disabled", $(this).attr("disabled"));
		}
	});

	$('.window-grid', element).each(function () {
		var grid = $(this);
		var table = $(".window-grid-table", $(this));
		var width = parseInt(table.attr("data-width"));
		table.jqGrid({
			width: width !== -1 ? width : null,
			shrinkToFit: width !== -1,
			datatype: "local",
			height: table.attr("data-height"),
			cellEdit: true,
			colNames: (function () {
				var cols = new Array();
				$(".window-column", table.parent()).each(function () { cols.push($(this).attr("data-name")); });
				return cols;
			})(),
			colModel: (function () {
				var cols = new Array();
				$(".window-column", table.parent()).each(function () {
					var col = { name: $(this).attr("data-id"),
							    index: $(this).attr("data-id"),
							    width: $(this).attr("data-width"),
							    title: false, // Ei tooltippejä onmouseoverissa
								sortable: false
							  };
					if ($(this).attr("data-editable") === "true") {
						col.editable = true;
						col.edittype = $(this).attr("data-edittype");
						col.formatter = "select";
								
						var values = {};
						var opts = $(this).attr("data-editvalues").split("|");
						for (var i = 0; i < opts.length; ++i) values[i] = opts[i];

						col.editoptions = {value: values,
											dataEvents: [
											{type: 'change',
											 fn: function (e) {
												 // Rivinumero on select-elementin id:n alussa
												 var id = $(this).prop("id");
												 var row = id.slice(0, id.indexOf("_"));
												 table.setCell(row, col.name, this.value);
												 sendMessage(grid, "change", {"row": row - 1, "column": col.name, "value": this.value});
											 }},
										   ]};
					} else if ($(this).attr("data-button") === "true") {
						col.align = "center";
						col.formatter = function(cellvalue, options, rowobject) {
							return "<button class='window-grid-col-button' data-row=" + options.rowId + " type='button'>" + cellvalue + "</button>";
						}
					}
					cols.push(col);
				});
				
				return cols;
			})(),
			gridComplete: function () {
				if (table.attr("data-selectable") !== "true") table.jqGrid("hideCol", "cb");
			},
			multiselect: table.attr("data-multiselect") === "true"
		});
	});
	
	$('.window-motionmask', element).each(function () {
		var maskElement = $(this).parent().children(".window-motionmask-mask");
		var mask = maskElement.val();

		var cols = $(this).attr("data-maskwidth");
		var rows = $(this).attr("data-maskheight");

		var pixelWidth = $(this).attr("data-width") / cols;
		var pixelHeight = $(this).attr("data-height") / rows;

		var maskDiv = $("<div style='position: absolute; left: 0px; top: 0px'></div>");

		for (var row = 0; row < rows; ++row) {
			for (var col = 0; col < cols; ++col) {
				var pixel = $("<div class='window-motionmask-pixel'></div>");
				var index = row * cols + col;
				pixel.attr("data-index", index);
				pixel.css({'opacity': mask[index] === "1" ? 0.5 : 0,
							'background-color': 'red',
							'position': 'absolute',
							'left': col * pixelWidth + "px",
							'top': row * pixelHeight + "px",
							'height': pixelHeight + "px",
							'width': pixelWidth + "px"
				});
				pixel.mouseover(function (event) {
					if (event.which === 1 || event.which === 3) {
						var m = maskElement.val();
						var index = parseInt($(this).attr("data-index"));
						$(this).css("opacity", event.which === 1 ? 0.5 : 0);
						var m = maskElement.val();
						maskElement.attr("value", m.substr(0, index) + (event.which === 1 ? "1" : "0") + m.substr(index + 1));
					}
				});
				maskDiv.append(pixel);
			}
		}
		
		$(this).append(maskDiv);
	});
	
	$(".window-camerapanel", element).each(function () {
		var areas = [];
		$(".window-camerapanel-area", this).each(function () {
			var area = new Map();
			area["id"] = $(this).attr("data-id");
			area["type"] = $(this).attr("data-type");
			area["title"] = $(this).attr("data-title");
			
			var areaPoints = [];
			var splitted = $(this).attr("data-points").split(",");
			for (var i = 0; i <= splitted.length - 2; i += 2) areaPoints.push([parseFloat(splitted[i]), parseFloat(splitted[i + 1])]);
			area["points"] = areaPoints;
			
			areas.push(area);		
		});

		var panel = $(this).camerapanel({
			camera: {
				label: $(this).attr("data-label"),
                uuid: $(this).attr("data-name"),
			},
			width: $(this).width(),
			height: 400,
			rtspMode: false,
			rtspPort: 554,
			viewId: 0,
			windowless: false,
			selectedArea: $(this).attr("data-selectedarea"),

			areaselected: function () {
				sendMessage($(this), "custom", {"type": "areaselected", "value": $(this).camerapanel("selectedArea")});
			},
			arearemoved: function (event, areaId) {
				sendMessage($(this), "custom", {"type": "arearemoved", "value": areaId});
			},
			areacreated: function (event, area) {
				sendMessage($(this), "custom", {"type": "areacreated", "value": area});
			}
		});
		$(this).camerapanel("option", "mode", $(this).attr("data-mode"));
		
		$(this).camerapanel("setMask", $(this).attr("data-mask"));
		$(this).bind("camerapanelmaskchanged", function () {
			sendMessage($(this), "custom", {"type": "maskchanged", "value": $(this).camerapanel("mask")});
		});
		$(this).bind("camerapanelareaschanged", function () {
			sendMessage($(this), "custom", {"type": "areaschanged", "value": $(this).camerapanel("areas")});
		});
		
		for (var i = 0; i < areas.length; ++i) $(this).camerapanel("addArea", areas[i]);
	});
};

function windowLoaded() {
	$('.window[data-type="dialog"]').each(function () {
		if ($(this).attr("data-fullscreen") === "true") {
			$(this).css("position", "absolute").css("top", "0px").css("left", "0px").css("width", "100%").css("height", "100%").css("z-index", 1000);
			$(this).parent().parent().show();
			$(".content-area", $(this)).css("position", "absolute").css("width", $(this).attr("data-width") + "px").css("left", "50%").css("top", "50%").css("transform", "translateY(-50%),translateX(-50%)");
		} else {
			var id = "dialog-" + $(this).attr("data-id");
			$(this).parent().append("<div id='" + id + "'></div>");
			$(this).dialog({
				appendTo: "#" + id,
				closeOnEscape: $(this).attr("data-closeOnEscape") !== "false",
				dialogClass: $(this).attr("data-showCloseButton") === "false" ? "no-close" : "",
				height: $(this).attr("data-height") === undefined ? 350 : $(this).attr("data-height"),
				width: $(this).attr("data-width") === undefined ? 400 : $(this).attr("data-width"),
				resizable: $(this).attr("data-resizable") === undefined ? true : $(this).attr("data-resizable") === "false" ? false : true,
				draggable: $(this).attr("data-draggable") === undefined ? true : $(this).attr("data-draggable") === "false" ? false : true,
				modal: true,
				position: { my: "center", at: "center" },
				autoOpen: true,
				title: $(this).attr("data-title"),
				close: function(event, ui) {
					// Lähetään closed-eventti vain kun käyttäjä on sulkenut ikkunan
					if ($(this).data("nonuserclose") !== "true") sendMessage($(this), "closed");
					$(this).dialog("destroy");
					$(this).parent().remove();
				},
				open: function(event, ui) {
					$(".ui-dialog-titlebar-close", ui.dialog | ui).focus();
				}
			});
		}
	});
	
	$('.window').each(function () {
		$('.window-element', $(this)).each(function () {
			onWindowElementReady($(this));
		});
		
		$('.window-row', $(this)).each(function () {
			$(this).children().addClass("window-row-item");
		});
		
		$('.window-content[data-hidden="true"]', $(this)).each(function () {
			if ($(this).parent().prop('nodeName') === "LI") $(this).parent().hide();
			else $(this).hide();
		});
	});
	
	$(".window-tooltip").qtip({style: {
		widget: true
	}});
	
	$(".ui-tabs").tabs({
		active: 0,
		heightStyle: "content"
	});
}

function rotate(element) {
	$({deg: 0}).animate({deg: 100000}, {
		duration: 100000,
		easing: "linear",
		step: function (now) { element.css({ transform: 'rotate(' + now + 'deg)' }); }
	});
}

function processWindowMessage(message) {
	var window = $(".window[data-id='" + message.window + "']");
	if (message.command === "close") {
		window.data("nonuserclose", "true");
		window.dialog("close");
	} else if (message.command === "hide") {
		window.parent().parent().hide();
	} else if (message.command === "show") {
		window.parent().parent().show();
		window.parent().show();
	} else if (message.command === "setbusy") {
		$(".window-busy", window).each(function () {
			rotate($(this));
			$(this).show();
		});
		$(".window-input:not(.ui-state-disabled)", window).each(function () {
			$(this).addClass("window-input-busy-disabled").addClass("ui-state-disabled").prop("disabled", true);
		});
	} else if (message.command === "setunbusy") {
		$(".window-busy", window).each(function () {
			$(this).hide();
		});
		$(".window-input-busy-disabled", window).each(function () {
			$(this).removeClass("window-input-busy-disabled").removeClass("ui-state-disabled").prop("disabled", false);
		});
	} else { // Tiettyyn elementtiin liittyvä komento
		var target = $("[data-id='" + message.element + "']", window);
		if (message.command === "hideelement") {
			if (target.parent().prop('nodeName') === "LI") target.parent().hide();
			else target.hide();
		} else if (message.command === "showelement") {
			if (target.parent().prop('nodeName') === "LI") target.parent().show();
			else target.show();
		} else if (message.command === "setelementbusy") {
			var busy = $(".window-busy", target);
			busy.show();
			rotate(busy);
		} else if (message.command === "setelementunbusy") {
			$(".window-busy", target).hide();
		} else if (message.command === "enableelement") {
			target.removeClass("ui-state-disabled").prop("disabled", false);
		} else if (message.command === "disableelement") {
			target.addClass("ui-state-disabled").prop("disabled", true);
		} else if (message.command === "setelementstate") {
			if (target.hasClass("window-text")) {
				target.removeClass("ui-state-error-text ui-state-success-text");
				target.addClass("ui-state-" + message.state + "-text");
			} else {
				target.removeClass("ui-state-error ui-state-highlight ui-state-success");
				target.addClass("ui-state-" + message.state);
			}
		} else { // update-komento
			var update = message;
			if (target.hasClass("window-camerapanel")) {
				if (update.attr === "status") target.camerapanel("updateStatus", update.value);
				else if (update.attr === "recordingStatus") target.camerapanel("updateRecordingStatus", update.value === "true");
				else if (update.attr === "frame") target.camerapanel("update", update.value.length === 0 ? "" : "data:image/jpeg;base64," + update.value);
				else if (update.attr === "mode") target.camerapanel("option", "mode", update.value);
				else if (update.attr === "error") target.camerapanel("option", "error", update.value);
				else if (update.attr === "maskColor") target.camerapanel("setMaskColor", update.value);
				else if (update.attr === "mask") target.camerapanel("setMask", update.value);
				else if (update.attr === "motion") target.camerapanel("setMotion", update.value);
			} else if (target.is("select") && !update.hasOwnProperty("attr")) {
				target.find('option').remove();
				var options = eval(update.options);
				if (options === null) return;
					
				for (var m = 0; m < options.length; m++) {
					var option = options[m];
					var element = $("<option></option>");
					element.attr("value", option.id);
					element.text(option.name);
					if (option.disabled === "true") {
						element.addClass("ui-state-disabled");
						element.prop("disabled", true);
					}
					target.append(element);
				}
			} else if (target.hasClass("window-progressbar")) {
				target.progressbar("option", update.attr, isNaN(parseInt(update.value)) ? (update.value === "false" ? false : true) : parseInt(update.value));
			} else if (target.hasClass("window-link")) {
				if (update.hasOwnProperty("attr") && update.attr === "url") target.children("a").attr("href", update.value);
				if (update.hasOwnProperty("text")) target.children("a").text(update.text);
			} else if (target.hasClass("window-grid")) {
				var table = $(".window-grid-table", target);
				table.jqGrid("clearGridData");
				var rows = eval(update.rows);
				if (typeof rows === "undefined" || rows === null) return;
				for (var m = 0; m < rows.length; m++) {
					table.jqGrid('addRowData', m + 1, eval(rows[m].values));
					if (rows[m].disabled === "true") {
						var e = $("tr#" + (m + 1) + "> td > input", table); // Vaan editoitavissa riveissä on input-elementti
						if (e.length > 0) e.attr("disabled", "true");
						else $("tr#" + (m + 1) + "> td", table).addClass("ui-state-disabled");
					}
					if (rows[m].error === "true") {
						$("tr#" + (m + 1) + "> td", table).addClass("ui-state-error");
					}
				}
			} else if (target.hasClass("window-number") && update.attr === "label") {
				target.prev().text(update.value);
			} else if (target.hasClass("window-content")) {
				if (update.attr === "icon") $(".window-content-title-icon", target).css("background-image", "url('" + update.value + "')");
				else if (update.attr === "title") $(".window-content-title-text", target).html(update.value);
				else {
					target.html(update.text);
					$('.window-element', target).each(function () {
						onWindowElementReady($(this));
					});
					$('.window-row', target).each(function () {
						$(this).children().addClass("window-row-item");
					});
				}
			} else {
				// Attribuuttia ja valueta ei voida koskaan muuttaa samassa viestissä
				if (update.hasOwnProperty("attr") && update.attr !== "") target.attr(update.attr, update.value);
				else if (update.hasOwnProperty("value")) target.val(update.value);
				
				if (update.hasOwnProperty("text")) target.text(update.text);
			}
		}
	}
}

/*
Palauttaa select-elementin valitut optionit pilkuilla eroteltuina.
Jos vain yksi valittu optioni, palautetaan se.
*/
function getOptions(select) {
	var s = "";
	var opts = select.children("option:selected");
	if (opts.length) {
		opts.each(function () {
			s = s + $(this).val() + ",";
		});
		s = s.slice(0, -1); // poistetaan viimeinen pilkku
	}
	return s;
}

function submit(element) {
	var content = element.closest(".window-content");

	var values = {};
	$(".window-input", content).each(function () { // lisätään kaikki window-input -luokkaan kuuluvien elementtien arvot mukaan
		var key = $(this).attr("data-name");
		if ($(this).prop("tagName") === "SELECT") values[key] = getOptions($(this)); // Input-elementti oli select-tyyppinen, lisätään valitut optionit
		else values[key] = $(this).val();
	});
	$(".window-grid", content).each(function () {
		// Input-elementti on grid, haetaan valitut rivit ja lisätään ne
		var array = new Array();
		var grid = $(".window-grid-table", this);
		var selectedIds = grid.jqGrid("getGridParam", "selarrrow");
		for (var i = 0; i < selectedIds.length; ++i) array.push(grid.getLocalRow(selectedIds[i]));
		values[$(this).attr("data-name")] = array;
	});
	$(".window-scheduleeditor", content).each(function () { values[$(this).attr("data-name")] = $(this).attr("data-value"); });
	
	sendMessage(content, "submit", values);
}

$(document).ready( function() {
	$(document).on('keypress', '.window-item[data-controltype="submit"]', function (event) {
		if (event.which !== 13) return; // 13 on entteri
		submit($(this));
		return false;
	});

	// Password- ja text-tyyppisten elementtien submitin hyväksyminen tapahtuu entterillä
	$(document).on("click", '.window-item:not(select)[data-controltype="submit"][type!="password"][type!="text"]', function () {	
		submit($(this));
		return false; // palautetaan false, jotta eventin käsittely loppuu
	});
	
	$(document).on("change", 'select.window-item[data-controltype="submit"]', function () {
		submit($(this));
		return false;
	});
	
	$(document).on("click", '.window-item[data-controltype="close"]', function () {
		sendMessage($(this), "click");
		$(this).closest(".window").dialog("close");
		return false;
	});
	
	$(document).on("click", '.window-item[data-controltype="refresh"]', function () {
		location.reload(true);
		return false;
	});

	$(document).on("click", ".window-topic-help", function () {
		sendMessage($(this), "click");
		return false;
	});

	$(document).on("click", '.window-button', function () {
		if ($(this).attr("data-controltype") !== "close" && $(this).attr("data-controltype") !== "submit"
		 && $(this).attr("data-controltype") !== "refresh") { // close-, submit- ja refresh-komennot käsitellään muualla
			sendMessage($(this), "click");
			return false;
		}
		return true;
	});
	
	$(document).on("click", '.window-grid-col-button', function () {
		sendMessage($(this).closest(".window-grid"), "click", $(this).attr("data-row") - 1);
		return true;
	});
	
	$(document).on("change", ".window-input", function () {
		if ($(this).attr("data-silent") === "true") return;
		
		var value = "";
		if ($(this).attr("type") === "checkbox") {
			value = $(this).prop("checked");
		} else if (typeof $(this).attr("list") !== undefined || $(this).attr("list") === true) {
			var datalist = $("#" + $(this).attr("list"));
			var option = datalist.find("option[value='" + value + "']")[0]; // Jos ei löydy optioinia, on kyseessä käyttäjän syöttämä teksti
			value = typeof option !== "undefined" ? $(option).attr("data-id") : $(this).val();
		} else {
			value = $(this).val();
		}
		
		sendMessage($(this), "change", value);
	});

	$(document).on("input", ".window-text-input[data-constantchangeupdates=true]", function() {
		if ($(this).attr("data-silent") !== "true") sendMessage($(this), "change", $(this).val());
    });

	$(document).on("click", ".window-file-submit", function () {
		var fileInput = $(".window-file", $(this).parent())[0];
		if (fileInput.files.length === 0) return;

		var fileSubmit = $(this);
		var reader = new FileReader;
		reader.onloadend = function (bytes) {
			function arrayBufferToBase64(buffer) {
				var binary = '';
				var bytes = new Uint8Array(buffer);
				var len = bytes.byteLength;
				for (var i = 0; i < len; i++) binary += String.fromCharCode(bytes[i]);
				return window.btoa(binary);
			};
			
			var values = {};
			values[fileSubmit.attr("data-id")] = arrayBufferToBase64(reader.result);

			sendMessage(fileSubmit, "submit", values);
		}
		reader.readAsArrayBuffer(fileInput.files[0]);
	});

	$(document).on("change", ".window-file", function () {
		var fileSubmit = $(".window-file-submit", $(this).parent().parent());
		fileSubmit.prop("disabled", this.files.length === 0);
		fileSubmit.toggleClass("ui-state-disabled", this.files.length === 0);
	});
	
	$('input.cbox[disabled=disabled]').livequery(function () {
		$(this).parent().parent().addClass("ui-state-disabled");
	});

	$(document).on("click", ".window-schedule-editor-item", function() {
		function setCharAt(str, index, chr) {
			if (index > str.length-1) return str;
			return str.substr(0,index) + chr + str.substr(index+1);
		}
		var selected = $(this).attr("data-selected") === "true";
		$(this).attr("data-selected", selected ? "false" : "true");
		var editor = $(this).closest(".window-scheduleeditor");
		value = setCharAt(editor.attr("data-value"), parseInt($(this).attr("data-hour")) * 7 + parseInt($(this).attr("data-day")), (!selected ? 'A' : 'a'));
		editor.attr("data-value", value);
	});

	$(document).on("click", ".window-schedule-editor-hour", function () {
		var hour = $(this).attr("data-hour");
		$(".window-schedule-editor-item[data-hour='" + hour + "']", $(this).closest(".window-scheduleeditor")).each(function () { $(this).click(); });
	});

	$(document).on("click", ".window-schedule-editor-day", function () {
		var day = $(this).attr("data-day");
		$(".window-schedule-editor-item[data-day='" + day + "']", $(this).closest(".window-scheduleeditor")).each(function () { $(this).click(); });
	});
	
	$(document).on("change", ".window-multichoice-choice", function () {
		sendMessage($(this).closest(".window-item"), "change", {"id": $(this).attr("value"), "selected": $(this).prop("checked") ? 1 : 0});
	});
});
